"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Csv {
    constructor(output) {
        this.fields = {
            name: 'Bucket name',
            BlockPublicAcls: 'BlockPublicAcls is enabled',
            BlockPublicPolicy: 'BlockPublicPolicy is enabled',
            RestrictPublicBuckets: 'RestrictPublicBuckets is enabled',
            IgnorePublicAcls: 'IgnorePublicAcls is enabled',
            sse: 'Server side encryption algorith',
            versioning: 'Object versioning is enabled',
            mfa_delete: 'MFA Delete is enabled',
            static_website: 'Static website hosting is disabled',
            bucket_policy: 'Bucket policy doesn\'t allow a wildcard entity',
            bucket_acl: 'Bucket ACL doesn\'t allow public access',
            logging: 'Bucket logging target bucket',
            cloudfront: 'CloudFront Origin Access Identities'
        };
        this.fieldsToChecks = {
            BlockPublicAcls: 'public-access-config',
            BlockPublicPolicy: 'public-access-config',
            RestrictPublicBuckets: 'public-access-config',
            IgnorePublicAcls: 'public-access-config',
            sse: 'sse',
            versioning: 'versioning',
            mfa_delete: 'mfa-delete',
            static_website: 'websited',
            bucket_policy: 'policy',
            bucket_acl: 'acl',
            logging: 'logging',
            cloudfront: 'cloudfront'
        };
        this.output = output;
    }
    async run(buckets, checks) {
        const includeFields = this.getIncludedFields(checks);
        const headings = includeFields.map((field) => this.fields[field]);
        this.output(headings.join(','));
        buckets.forEach(async (bucket) => {
            const bucketDetails = { name: bucket.name };
            Promise.all(this.getEnabledCheckPromises(checks, bucket, bucketDetails)).then(() => {
                const output = [];
                includeFields.forEach((field) => {
                    output.push(bucketDetails[field]);
                });
                this.output(output.join(','));
            });
        });
    }
    getEnabledCheckPromises(checks, bucket, bucketDetails) {
        const promises = [];
        if (checks.includes('public-access-config')) {
            promises.push(this.populatePublicAccessConfiguration(bucket, bucketDetails));
        }
        if (checks.includes('sse')) {
            promises.push(this.populateSSEField(bucket, bucketDetails));
        }
        if (checks.includes('logging')) {
            promises.push(this.populateLoggingField(bucket, bucketDetails));
        }
        if (checks.includes('versioning')) {
            promises.push(this.populateVersioningField(bucket, bucketDetails));
        }
        if (checks.includes('website')) {
            promises.push(this.populateStaticWebsiteField(bucket, bucketDetails));
        }
        if (checks.includes('policy')) {
            promises.push(this.populatePolicyField(bucket, bucketDetails));
        }
        if (checks.includes('acl')) {
            promises.push(this.populateACLField(bucket, bucketDetails));
        }
        if (checks.includes('mfa-delete')) {
            promises.push(this.populateMFADeleteField(bucket, bucketDetails));
        }
        if (checks.includes('cloudfront')) {
            promises.push(this.populateCloudfrontField(bucket, bucketDetails));
        }
        return promises;
    }
    getIncludedFields(checks) {
        return Object.keys(this.fields).filter((field) => {
            if (this.fieldsToChecks[field] === undefined) {
                return true;
            }
            return checks.includes(this.fieldsToChecks[field]);
        });
    }
    populatePublicAccessConfiguration(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getPublicAccessConfiguration()
                .then((bucketPublicAccessBlock) => {
                Object.assign(bucketDetails, bucketPublicAccessBlock);
            })
                .catch((error) => {
                Object.assign(bucketDetails, {
                    BlockPublicAcls: error.message,
                    BlockPublicPolicy: error.message,
                    RestrictPublicBuckets: error.message,
                    IgnorePublicAcls: error.message
                });
            })
                .finally(() => resolve());
        });
    }
    populateSSEField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getSSEAlgorithm()
                .then((algorithm) => {
                bucketDetails.sse = algorithm || 'Disabled';
            })
                .catch((error) => {
                bucketDetails.sse = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateLoggingField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getLoggingTargetBucket()
                .then((bucket) => {
                bucketDetails.logging = bucket || 'Disabled';
            })
                .catch((error) => {
                bucketDetails.logging = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateVersioningField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasVersioningEnabled()
                .then((isEnabled) => {
                bucketDetails.versioning = isEnabled;
            })
                .catch((error) => {
                bucketDetails.versioning = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateStaticWebsiteField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasStaticWebsiteHosting()
                .then((isEnabled) => {
                bucketDetails.static_website = !isEnabled;
            })
                .catch((error) => {
                bucketDetails.static_website = error.message;
            })
                .finally(() => resolve());
        });
    }
    populatePolicyField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getPolicyWildcardEntities()
                .then((entities) => {
                bucketDetails.bucket_policy = entities.length === 0;
            })
                .catch((error) => {
                bucketDetails.bucket_policy = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateACLField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.allowsPublicAccessViaACL()
                .then((isPublic) => {
                bucketDetails.bucket_acl = isPublic === false;
            })
                .catch((error) => {
                bucketDetails.bucket_acl = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateMFADeleteField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasMFADeleteEnabled()
                .then((isEnabled) => {
                bucketDetails.mfa_delete = isEnabled;
            })
                .catch((error) => {
                bucketDetails.mfa_delete = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateCloudfrontField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getCloudFrontOriginAccessIdentities()
                .then((identities) => {
                bucketDetails.cloudfront = identities.length === 0 ?
                    'None'
                    : identities.join(', ');
            })
                .catch((error) => {
                bucketDetails.cloudfront = error.message;
            })
                .finally(() => resolve());
        });
    }
}
exports.default = Csv;
