"use strict";
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const aws_sdk_1 = require("aws-sdk");
const bucket_1 = tslib_1.__importDefault(require("./bucket"));
const console_1 = tslib_1.__importDefault(require("./formatters/console"));
const csv_1 = tslib_1.__importDefault(require("./formatters/csv"));
class S3Audit extends command_1.Command {
    async run() {
        const { flags } = this.parse(S3Audit);
        const buckets = [];
        const format = flags.format;
        const includeChecks = this.determineChecksToRun(flags);
        if (flags.bucket) {
            return this.auditBuckets(format, [new bucket_1.default(flags.bucket)], includeChecks);
        }
        new aws_sdk_1.S3().listBuckets((error, data) => {
            if (!data || data.Buckets === undefined) {
                return this.exit();
            }
            data.Buckets.forEach(bucket => {
                if (bucket.Name !== undefined) {
                    buckets.push(new bucket_1.default(bucket.Name));
                }
            });
            this.auditBuckets(format, buckets, includeChecks);
        });
    }
    auditBuckets(format, buckets, includeChecks) {
        let formatter;
        switch (format) {
            case 'csv':
                formatter = new csv_1.default((output) => {
                    this.log(output);
                });
                break;
            default:
                formatter = new console_1.default();
        }
        formatter.run(buckets, includeChecks);
    }
    determineChecksToRun(flags) {
        let checks = S3Audit.checks;
        if (flags['enable-check']) {
            checks = flags['enable-check'];
        }
        if (flags['disable-check']) {
            checks = checks.filter((check) => flags['disable-check'].indexOf(check) === -1);
        }
        return checks;
    }
}
S3Audit.description = 'Audits S3 bucket settings';
S3Audit.args = [];
S3Audit.checks = [
    'public-access-config',
    'versioning',
    'sse',
    'mfa-delete',
    'website',
    'policy',
    'acl',
    'logging',
    'cloudfront'
];
S3Audit.examples = [
    '$ s3audit',
    '$ s3audit --bucket=s3-bucket=name',
    '$ s3audit --format=csv',
    '$ s3audit --enable-check=policy --enable-check=acl',
    '$ s3audit --disable-check=logging --enable-check=logging',
];
S3Audit.flags = {
    bucket: command_1.flags.string({
        description: 'The name of a bucket to target'
    }),
    'disable-check': command_1.flags.string({
        char: 'd',
        exclusive: ['enable-check'],
        multiple: true,
        options: S3Audit.checks
    }),
    'enable-check': command_1.flags.string({
        char: 'e',
        exclusive: ['disable-check'],
        multiple: true,
        options: S3Audit.checks
    }),
    format: command_1.flags.string({
        description: 'The output format to use',
        default: 'console',
        options: ['console', 'csv']
    }),
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
};
S3Audit.usage = 's3audit --bucket=s3-bucket-name --output=csv';
module.exports = S3Audit;
